"""
Models module initialization
"""

from .requests import (
    DiversifiedSearchRequest,
    MemoryDiscoverAssociationsRequest,
    MemoryExportRequest,
    MemoryImportRequest,
    MemoryListAllRequest,
    MemoryManageRequest,
    MemoryMoveRequest,
    MemorySearchRequest,
    MemoryStoreRequest,
    MemorySyncRequest,
    MemoryUpdateRequest,
    ScopeListRequest,
    ScopeSuggestRequest,
    SessionManageRequest,
    UnifiedSearchRequest,
)
from .responses import (
    Association,
    ErrorResponse,
    MCPResponse,
    Memory,
    MemoryDeleteResponse,
    MemoryDiscoverAssociationsResponse,
    MemoryExportResponse,
    MemoryGetResponse,
    MemoryImportResponse,
    MemoryListAllResponse,
    MemoryMoveResponse,
    MemoryResponse,
    MemorySearchResponse,
    MemoryStoreResponse,
    MemoryUpdateResponse,
    MemoryWithAssociations,
    PaginationInfo,
    ScopeInfo,
    ScopeListResponse,
    ScopeRecommendation,
    ScopeSuggestResponse,
    SearchResult,
    SearchResultWithAssociations,
    SessionInfo,
    SessionManageResponse,
)

__all__ = [  # Request models
    "MemoryStoreRequest",
    "MemorySearchRequest",
    "DiversifiedSearchRequest",
    "UnifiedSearchRequest",
    "MemoryManageRequest",
    "MemoryDiscoverAssociationsRequest",
    "MemoryListAllRequest",
    "MemorySyncRequest",
    "MemoryUpdateRequest",
    "MemoryMoveRequest",
    "ScopeListRequest",
    "ScopeSuggestRequest",
    "SessionManageRequest",
    "MemoryExportRequest",
    "MemoryImportRequest",
    # Response models
    "Memory",
    "SearchResult",
    "Association",
    "MemoryWithAssociations",
    "SearchResultWithAssociations",
    "ScopeInfo",
    "ScopeRecommendation",
    "SessionInfo",
    "PaginationInfo",
    "MemoryResponse",
    "MemoryStoreResponse",
    "MemorySearchResponse",
    "MemoryGetResponse",
    "MemoryUpdateResponse",
    "MemoryDeleteResponse",
    "MemoryMoveResponse",
    "MemoryListAllResponse",
    "MemoryDiscoverAssociationsResponse",
    "ScopeListResponse",
    "ScopeSuggestResponse",
    "SessionManageResponse",
    "MemoryExportResponse",
    "MemoryImportResponse",
    "ErrorResponse",
    "MCPResponse",
]
